(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This program is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This software is distributed in the hope that it will be useful,      *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(* main program *)

open ProuveParser

let prouve_output = ref false;;
let simplify_output = ref false;;
let inputfilename = ref "";;
let outputfilename = ref "";;
let version = "1.4.0";;

Arg.parse 
   [
     ("-o" , 
      Arg.String (function s ->
		    prouve_output := true;
		    outputfilename := s) ,
      "<file> Print the file in prouve syntax to <file>");
     ("-simplify" , 
      Arg.Unit (function () ->
		  prouve_output := true; simplify_output := true) ,
      "Print the file in prouve syntax after simplification (not yet implemented)");
     ("-version",
      Arg.Unit (function () -> 
		  print_string "PROUVE parser version " ;
		  print_string version;
		  print_newline ();
		  exit 0;
	       ),
      "Print version number")
   ]
  (function s -> inputfilename := s)
  "Parse a file containing a PROUV protocol specification or assertion.";
   
if !inputfilename = ""
then 
  begin
    prerr_string "Filename argument missing!";
    prerr_newline ();
    exit 1
  end
;;

if (String.length !inputfilename) < 5
then begin
  prerr_string "Filename has to have suffix \".prv\" or \".pra\"";
  prerr_newline ();
  exit 1
end
;;

if (String.sub !inputfilename ((String.length !inputfilename)-4) 4) = ".prv"
then (* specification file *)
  try
    let spec = AbstractSyntax.parse_spec !inputfilename
    in
      (try
	 print_string "Input specification file ";
	 print_string !inputfilename;
	 print_string " is syntactically correct.\n";
	 if !prouve_output then
	   let oc=open_out !outputfilename
	   in
	     begin
	       AbstractSyntax.print_spec oc spec;
	       print_string "PROUVE output written to file ";
	       print_string !outputfilename;
	     end;
       with
	   Sys_error s -> print_string ("Cannot open file " ^ !outputfilename ^
					  " for writing because " ^ s))
  with
      Exc_prouve s -> print_string("Error parsing specification " ^
				     !inputfilename ^ " because: " ^ s)
else if (String.sub !inputfilename ((String.length !inputfilename)-4) 4)
  = ".pra"
then (* assertion file *)
    try
      let assertion = Logic.parse_assertion !inputfilename
      in
	print_string "Input assertion file ";
	print_string !inputfilename;
	print_string " is syntactically correct.";
    with
	Exc_prouve s -> print_string("Error parsing specification " ^
				       !inputfilename ^ " because: " ^ s)
else begin
  prerr_string "Filename has to have suffix \".prv\" or \".pra\"";
  prerr_newline ();
  exit 1
end
  
;;

print_newline ()

	  
